/*-----------.
| Prologue.  |
`-----------*/

%{
#include <stdio.h>
#include <stdlib.h>

%}
%defines
%debug
%expect 0
%verbose
%error-verbose
%locations
%pure-parser
%param { size_t* errors }
%union
{
  int ival;
}
%printer { fprintf(stderr, "%d", $$); } <ival>;
%code provides
{
#define YY_DECL							\
  enum yytokentype						\
  yylex(YYSTYPE* yylval, YYLTYPE* yylloc, size_t* errors)
  YY_DECL;

  void yyerror(YYLTYPE* loc, size_t* errors, const char* msg);
}

%token <ival>
  INT "number"
;
%define api.tokens.prefix "TOK_"
%token
  PLUS "+" MINUS "-" 
  LPAR "(" RPAR ")"
  STAR "*" SLASH "/" 
  EOL "end of line"
;
%type <ival> exp line;

%left PLUS MINUS
%left STAR SLASH

%start input
%%
/*----------.
| Grammar.  |
`----------*/
input:
  /* empty. */ {}
| input line   {}
;

line:
  EOL            { $$ = 666; }
| exp EOL        { printf("%d\n", $exp); }
| error EOL      { yyerrok; }
;

exp:
  exp "+" exp   { $$ = $1 + $3; }
| exp "*" exp   { $$ = $1 * $3; }
| exp "/" exp   
  { 
    if ($3) 
      $$ = $1 / $3;
    else
      {
	yyerror(&@3, errors, "cannot divide by 0");
	YYERROR;
      }
  }
| exp "-" exp   { $$ = $1 - $3; }
| "(" exp ")"   { $$ = $2; }
| INT           { $$ = $1; }
;
%%
/*-----------.
| Epilogue.  |
`-----------*/
void
yyerror(YYLTYPE* loc, size_t* errors, const char* msg)
{
  ++*errors;
  YY_LOCATION_PRINT(stderr, *loc);
  fprintf(stderr, ": %s\n", msg);
}

int
main(int argc, const char *argv[])
{
  yydebug = !!getenv("YYDEBUG");
  size_t errors = 0;
  yyparse(&errors);
  return !!errors;
}
